//===========================================================================
//
//  Code written for CFA-10009 development board
//  (3.3v AtMega2561 at 8MHz)
//
//  CRYSTALFONTZ CFAL12832DB 128x23 OLED in I2C mode
//
//  ref: https://www.crystalfontz.com/product/cfal12832db
//
//  2016 - 04 - 13 Brent A. Crosby
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//===========================================================================

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "readkeys.h"
//============================================================================
// These are a bit-fields containing the keys that were pressed in the last
// poll.
volatile uint8_t keys_pressed_last_time;
volatile uint8_t presses_needing_service;
volatile uint8_t releases_needing_service;

//----------------------------------------------------------------------------
#define KP_CANCEL 0x10    //PD4 / SW1
#define KP_PLUS   0x20    //PD5 / SW2
#define KP_MINUS  0x40    //PD6 / SW3
#define KP_ENTER  0x80    //PD7 / SW4
//============================================================================
// ref: https://startingelectronics.org/tutorials/AVR-8-microcontrollers/ATtiny2313-tutorial/P9-TC0-interrupt-timer/
// interrupt service routine (ISR) for timer 0 A compare match
// ~ 3.4uS
//----------------------------------------------------------------------------
ISR (TIMER0_COMPA_vect)
{
	register uint8_t this_key_reading;

	//Read the port, mask the bits that are not connected to keys.
	//Keys are normally pulled high, they are grounded when pressed,
	//invert so pressed keys are ones.
	this_key_reading = (~PIND) & (KP_CANCEL | KP_PLUS | KP_MINUS |KP_ENTER);

	//Detect the presses.
	//Now this_key_reading has bits set for any keys that are pressed.
	//(this_key_reading^keys_pressed_last_time) will have ones in it for all
	//the keys that have changed. If we and that with keys_pressed_last_time,
	//which has ones in it for all the keys that changed and are now pressed,
	//(because they were high last reading) then we will have the keys that
	//need serviced.
	presses_needing_service|=
		(this_key_reading&(this_key_reading^keys_pressed_last_time));
	//This detects releases
	releases_needing_service|=
		(keys_pressed_last_time&(this_key_reading^keys_pressed_last_time));
	//Remember what happened.
	keys_pressed_last_time=this_key_reading;
}
//============================================================================
void key_init(void)
{
	keys_pressed_last_time = 0;
	presses_needing_service = 0;
	releases_needing_service = 0;

	//Set up ~32Hz timer for polling keys
	// Enable Interrupt TimerCounter0 Compare Match A (SIG_OUTPUT_COMPARE0A)
	TIMSK0 = _BV(OCIE0A);
	// Mode = CTC
	TCCR0A = _BV(WGM01);
	// Clock/1024 = 8MHz/1024 = 7,812.5Hx => 0.000128 seconds per tick
	TCCR0B = _BV(CS02) | _BV(CS00);
	// 0.000128*244 ~= 32Hz rep rate for SIG_OUTPUT_COMPARE0A
	OCR0A = 244;

	//the calling code needs to enable interrupts
}
//============================================================================
uint8_t getkey(void)
{
	//Presses
	if(presses_needing_service&KP_CANCEL)
	{
		presses_needing_service&=~KP_CANCEL;
		return(KEY_CANCEL_PRESS);
	}
	if(presses_needing_service&KP_PLUS)
	{
		presses_needing_service&=~KP_PLUS;
		return(KEY_PLUS_PRESS);
	}
	if(presses_needing_service&KP_MINUS)
	{
		presses_needing_service&=~KP_MINUS;
		return(KEY_MINUS_PRESS);
	}
	if(presses_needing_service&KP_ENTER)
	{
		presses_needing_service&=~KP_ENTER;
		return(KEY_ENTER_PRESS);
	}

	//Releases
	if(releases_needing_service&KP_CANCEL)
	{
		releases_needing_service&=~KP_CANCEL;
		return(KEY_CANCEL_RELEASE);
	}
	if(releases_needing_service&KP_PLUS)
	{
		releases_needing_service&=~KP_PLUS;
		return(KEY_PLUS_RELEASE);
	}
	if(releases_needing_service&KP_MINUS)
	{
		releases_needing_service&=~KP_MINUS;
		return(KEY_MINUS_RELEASE);
	}
	if(releases_needing_service&KP_ENTER)
	{
		releases_needing_service&=~KP_ENTER;
		return(KEY_ENTER_RELEASE);
	}
	return(KEY_NONE);
}
//============================================================================
uint8_t any_keys_waiting(void)
{
	return(presses_needing_service||releases_needing_service);
}
//============================================================================
